/*
Copyright (C) 2011 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package soleil;

public class Vecteur {
    double[] v;
    
    public Vecteur(final int dimensions) {
        v = new double[dimensions];
    }
    
    public Vecteur(final double v1, final double v2, final double v3) {
        v = new double[3];
        v[0] = v1;
        v[1] = v2;
        v[2] = v3;
    }
    
    public int getDimensions() {
        return(v.length);
    }
    
    public double get(final int pos) {
        return(v[pos]);
    }
    
    public void set(final int pos, final double valeur) {
        v[pos] = valeur;
    }
    
    // coordonnes sphriques (r, theta, phi) vers cartsiennes (x, y, z)
    public Vecteur spheriquesVersCartesiennes() {
        if (v.length != 3) {
            System.err.println("Conversion impossible de coordonnes (" + v.length + " != 3)");
            return(null);
        }
        final Vecteur res = new Vecteur(3);
        res.set(0, v[0] * Math.sin(v[1]) * Math.cos(v[2]));
        res.set(1, v[0] * Math.sin(v[1]) * Math.sin(v[2]));
        res.set(2, v[0] * Math.cos(v[1]));
        return(res);
    }
    
    // coordonnes cartsiennes (x, y, z) vers sphriques (r, theta, phi)
    public Vecteur cartesiennesVersSpheriques() {
        if (v.length != 3) {
            System.err.println("Conversion impossible de coordonnes (" + v.length + " != 3)");
            return(null);
        }
        final Vecteur res = new Vecteur(3);
        final double r = Math.sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
        res.set(0, r);
        res.set(1, Math.acos(v[2]/r));
        if (v[0] > 0) // x > 0
            res.set(2, Math.atan(v[1]/v[0]));
        else if (v[0] < 0) // x < 0
            res.set(2, Math.PI + Math.atan(v[1]/v[0]));
        else if (v[1] > 0) // x = 0, y > 0
            res.set(2, Math.PI / 2);
        else if (v[1] < 0) // x = 0, y < 0
            res.set(2, 3 * Math.PI / 2);
        else // x = 0, y = 0 : peu importe !
            res.set(2, 0);
        return(res);
    }
}
