/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

if (!Math.toRadians) {
    Math.toRadians = function(x) {
        return(x * Math.PI / 180);
    }
}
if (!Math.toDegrees) {
    Math.toDegrees = function(x) {
        return(x * 180 / Math.PI);
    }
}

var canvas;
var context;
var minDelta = -23.5 * (Math.PI/180);
var maxDelta = 23.5 * (Math.PI/180);
var minPhi = -90 * (Math.PI/180);
var maxPhi = 90 * (Math.PI/180);
var H = Math.PI / 3; // angle horaire
var deltaInitial = 15 * (Math.PI/180);
var phiInitial = 50 * (Math.PI/180);
var couleurEquateur = 'rgba(150, 255, 200, 0.8)';
var couleurTexteEquateur = 'rgb(100, 200, 150)';
var couleurHorizon = 'rgba(250, 210, 170, 0.8)';
var couleurTexteHorizon = 'rgb(200, 150, 100)';
var couleurSoleil = 'rgb(255, 255, 0)';
var couleurSoleilCouche = 'rgb(30, 30, 30)';
var couleurFlecheSoleil = 'rgb(255, 150, 0)';
var couleurPositionsSoleil = 'rgb(255, 100, 0)';
var couleurDelta = 'rgb(255, 50, 50)';
var couleurPhi = 'rgb(50, 50, 255)';
var couleurArcs = 'lightgray';
var couleurAngleA = 'rgb(150, 100, 50)';
var couleurAngleH = 'rgb(50, 150, 100)';
var couleurAngleHauteur = 'rgb(150, 50, 150)';
var precisionCourbes = 100;
var imgdimx;
var imgdimy;
var sliderDelta, sliderPhi;
var delta;
var phi;
var cx, cy; // position du centre de l'image en pixels
var rayon; // rayon du grand cercle en pixels
var labelDelta, labelPhi;
var rotObservateur; // matrice de rotation par rapport à l'observateur
var rotEquateur; // matrice de rotation de l'équateur céleste par rapport à l'horizon
var rotObsEq; // rotObservateur * rotEquateur
var O, Nord, Sud, Est, Ouest, Z; // points en coordonnées sphériques
var titreDelta, titrePhi;
var idTimeout = 0;

function init() {
    canvas = document.getElementById('dessin');
    var largeur_fenetre = 'innerWidth' in window ? window.innerWidth : document.documentElement.offsetWidth;
    var hauteur_fenetre = 'innerHeight' in window ? window.innerHeight : document.documentElement.offsetHeight;
    imgdimx = Math.min(largeur_fenetre - 10, hauteur_fenetre - 140);
    if (imgdimx <= 0)
        imgdimx = 500;
    imgdimy = imgdimx;
    canvas.width = imgdimx;
    canvas.height = imgdimy;
    context = canvas.getContext('2d');
    context.translate(0.5, 0.5);
    context.font = 'bold 12px sans-serif';
    
    delta = deltaInitial;
    titreDelta = '\u03B4';
    labelDelta = document.getElementById('valeurDelta');
    setLabelText(labelDelta, titreDelta + " = " + affichageAngleDMS(delta, false, false));
    labelDelta.style.color = couleurDelta;
    sliderDelta = new Curseur('curseurDelta', Math.toDegrees(minDelta), Math.toDegrees(maxDelta), Math.toDegrees(delta), 101, false, modifCurseur);
    
    phi = phiInitial;
    titrePhi = '\u03C6';
    labelPhi = document.getElementById('valeurPhi');
    setLabelText(labelPhi, titrePhi + " = " + affichageAngleDMS(phi, false, false));
    labelPhi.style.color = couleurPhi;
    sliderPhi = new Curseur('curseurPhi', -90, 90, Math.round(180 * (phi - minPhi) / (maxPhi - minPhi) - 90), 181, false, modifCurseur);
    
    rotObservateur = new Matrice(3, 3);
    rotEquateur = new Matrice(3, 3);
    O = new Vecteur3D(0, 0, 0);
    Nord = new Vecteur3D(1, Math.PI / 2, -Math.PI / 2);
    Sud = new Vecteur3D(1, Math.PI / 2, Math.PI / 2);
    Est = new Vecteur3D(1, Math.PI / 2, Math.PI);
    Ouest = new Vecteur3D(1, Math.PI / 2, 0);
    Z = new Vecteur3D(1, 0, 0);
    
    majCroquis();
}

function setLabelText(label, texte) {
    var first = label.firstChild;
    if (first == null) {
        label.appendChild(document.createTextNode(texte));
        return;
    } else
        first.nodeValue = texte;
}

function modifCurseur(e) {
    var id = e.target.id;
    if (id == 'curseurDelta') {
        var valeur = sliderDelta.getValeur();
        delta = Math.toRadians(valeur);
        setLabelText(labelDelta, titreDelta + ' = ' + affichageAngleDMS(delta, false, false));
    } else if (id == 'curseurPhi') {
        var valeur = sliderPhi.getValeur();
        phi = Math.toRadians(valeur);
        setLabelText(labelPhi, titrePhi + ' = ' + affichageAngleDMS(phi, false, false));
    }
    if (idTimeout != 0)
        clearTimeout(idTimeout);
    idTimeout = setTimeout(majCroquis, 0);
}

function majCroquis() {
    var alpha = 0.2; // angle d'observation
    rotObservateur.set(0, 0, Math.cos(alpha));
    rotObservateur.set(0, 1, 0);
    rotObservateur.set(0, 2, -Math.sin(alpha));
    rotObservateur.set(1, 0, 0);
    rotObservateur.set(1, 1, 1);
    rotObservateur.set(1, 2, 0);
    rotObservateur.set(2, 0, Math.sin(alpha));
    rotObservateur.set(2, 1, 0);
    rotObservateur.set(2, 2, Math.cos(alpha));
    
    var cphi = Math.PI / 2 - phi; // angle de l'équateur par rapport à l'horizon
    rotEquateur.set(0, 0, 1);
    rotEquateur.set(0, 1, 0);
    rotEquateur.set(0, 2, 0);
    rotEquateur.set(1, 0, 0);
    rotEquateur.set(1, 1, Math.cos(cphi));
    rotEquateur.set(1, 2, Math.sin(cphi));
    rotEquateur.set(2, 0, 0);
    rotEquateur.set(2, 1, -Math.sin(cphi));
    rotEquateur.set(2, 2, Math.cos(cphi));
    
    rotObsEq = rotObservateur.produitM(rotEquateur);
    
    context.fillStyle = 'white';
    context.fillRect(0, 0, imgdimx, imgdimy);
    cx = imgdimx / 2;
    cy = imgdimy / 2;
    rayon = Math.min(cx - 20, cy - 20); // rayon de la sphère en pixels
    
    context.beginPath();
    context.fillStyle = 'rgb(230, 230, 255)';
    context.arc(cx, cy, rayon, 0, 2*Math.PI, false);
    context.fill();
    
    context.strokeStyle = '#707070';
    context.stroke();
    
    var S = new Vecteur3D(1, Math.PI / 2 - delta, Math.PI / 2 - H); // position du soleil dans le repère de l'équateur
    var Scart = S.spheriquesVersCartesiennes();
    var ScartHoz = rotEquateur.produitV(Scart); // rotation équateur -> horizon
    var SsphHoz = ScartHoz.cartesiennesVersSpheriques(); // coordonnées sphériques de S dans le repère de l'horizon
    var ptS = conversion(S, true);
    var ptO = new Point2D(cx, cy);
    var M = new Vecteur3D(1, Math.PI / 2, Math.PI / 2 - H); // repère équatorial
    var ptM = conversion(M, true);
    var ptNord = conversion(Nord, false);
    var P = new Vecteur3D(1, Math.PI / 2 - phi, -Math.PI / 2);
    var ptP = conversion(P, false);
    var N = new Vecteur3D(1, Math.PI / 2, SsphHoz.get(2)); // repère de l'horizon
    var pt;
    
    // positions du soleil (derrière)
    dessinPositionsSoleil(false);
    
    // calcul et affichage cercle de l'équateur céleste (derrière)
    var pt;
    context.beginPath();
    pt = conversion(new Vecteur3D(1, Math.PI / 2, 0), true);
    context.moveTo(pt.x, pt.y);
    for (var i=1; i<=precisionCourbes; i++) {
        pt = conversion(new Vecteur3D(1, Math.PI / 2, - (i / precisionCourbes) * Math.PI), true);
        context.lineTo(pt.x, pt.y);
    }
    context.closePath();
    context.fillStyle = couleurEquateur;
    context.fill();
    
    dessinHorizon();
    
    // calcul cercle de l'équateur céleste (devant)
    context.beginPath();
    pt = conversion(new Vecteur3D(1, Math.PI / 2, 0), true);
    context.moveTo(pt.x, pt.y);
    for (var i=1; i<=precisionCourbes/2; i++) {
        pt = conversion(new Vecteur3D(1, Math.PI / 2, (i / Math.floor(precisionCourbes/2)) * Math.PI), true);
        context.lineTo(pt.x, pt.y);
    }
    context.closePath();
    // affichage cercle de l'équateur céleste (devant)
    context.fillStyle = couleurEquateur;
    context.fill();
    context.fillStyle = couleurTexteEquateur;
    if (phi > 0) {
        pt = conversion(Ouest, false);
        context.save();
        context.translate(pt.x, pt.y);
        context.rotate(phi - Math.PI/2);
        context.translate(-pt.x, -pt.y);
        context.fillText('Equateur céleste', pt.x - rayon * 0.75, pt.y + 10);
        context.restore();
    } else {
        pt = conversion(Est, false);
        context.save();
        context.translate(pt.x, pt.y);
        context.rotate(phi + Math.PI/2);
        context.translate(-pt.x, -pt.y);
        context.fillText('Equateur céleste', pt.x - rayon * 0.75, pt.y - 5);
        context.restore();
    }
    
    // positions du soleil (devant)
    dessinPositionsSoleil(true);
    
    // soleil
    dessinSoleil(S, ptS, SsphHoz);
    
    // arc ZSN
    dessinArcZSN(Z, SsphHoz, N);
    
    // arc PSM
    dessinArcPSM(P, S, M);
    
    dessinAngleDelta(M, S);
    
    dessinAnglePhi(P);
    
    // angle A (azimuth du Soleil)
    dessinAngleA(N);
    
    // angle H (angle horaire)
    dessinAngleH(M);
    
    // angle h (hauteur du soleil)
    dessinAngleHauteur(N, SsphHoz);
    
    // points
    context.fillStyle = 'black';
    context.strokeStyle = 'black';
    
    // affichage des points fixes
    context.font = '12px sans-serif';
    afficherPoint(O, false, true, "O");
    afficherPoint(Nord, false, true, "Nord");
    afficherPoint(Sud, false, true, "Sud");
    afficherPoint(Est, false, true, "Est");
    afficherPoint(Ouest, false, true, "Ouest");
    afficherPoint(Z, false, true, "Z");
    
    // affichage des points en mouvement
    context.font = 'bold 12px sans-serif';
    afficherPoint(P, false, true, "P");
    
    afficherPoint(M, true, true, "M");
    afficherPoint(N, false, true, "N");
    afficherPoint(S, true, true, "S");
    
    idTimeout = 0;
}

function dessinHorizon() {
    context.beginPath();
    var pt = conversion(new Vecteur3D(1, Math.PI / 2, 0), false);
    context.moveTo(pt.x, pt.y);
    for (var i=1; i<precisionCourbes/2; i++) {
        pt = conversion(new Vecteur3D(1, Math.PI / 2, (i / Math.floor(precisionCourbes/2)) * (2 * Math.PI)), false);
        context.lineTo(pt.x, pt.y);
    }
    context.closePath();
    context.fillStyle = couleurHorizon;
    context.fill();
    context.fillStyle = couleurTexteHorizon;
    pt = conversion(Ouest, false);
    context.save();
    context.translate(pt.x, pt.y);
    context.rotate(-0.1);
    context.translate(-pt.x, -pt.y);
    context.fillText('Horizon céleste', pt.x + rayon / 4, pt.y + 20);
    context.restore();
}

//protected void dessinPositionsSoleil(final boolean devant) {
function dessinPositionsSoleil(devant) {
    context.fillStyle = couleurPositionsSoleil;
    var pt;
    for (var i=0; i<precisionCourbes; i++) {
        pt = conversion(new Vecteur3D(1, Math.PI / 2 - delta, (i / precisionCourbes) *  (2 * Math.PI)), true);
        if ((i <= precisionCourbes/4 || i >= (precisionCourbes*3)/4) == devant) {
            context.beginPath();
            context.arc(pt.x, pt.y, 1, 0, 2*Math.PI, false);
            context.fill();
        }
    }
}

//protected void dessinSoleil(final Vecteur S, final Point2D ptS, final Vecteur SsphHoz) {
function dessinSoleil(S, ptS, SsphHoz) {
    if (SsphHoz.get(1) <= Math.PI/2)
        context.fillStyle = couleurSoleil;
    else
        context.fillStyle = couleurSoleilCouche;
    context.beginPath();
    context.arc(ptS.x, ptS.y, 5, 0, 2*Math.PI, false);
    context.fill();
    // flêche de direction du soleil
    var S2 = new Vecteur3D(1, S.get(1), S.get(2) - Math.PI/16);
    context.strokeStyle = couleurFlecheSoleil;
    dessinArc(S, S2, true, true);
}

//protected void dessinArcZSN(final Vecteur Z, final Vecteur SsphHoz, final Vecteur N) {
function dessinArcZSN(Z, SsphHoz, N) {
    context.strokeStyle = couleurArcs;
    var Zbis = new Vecteur3D(1, 0, SsphHoz.get(2)); // Z avec le même phi que SsphHoz
    dessinArc(Zbis, SsphHoz.get(1) < N.get(1) ? SsphHoz : N, false, false);
}

//protected void dessinArcPSM(final Vecteur P, final Vecteur S, final Vecteur M) {
function dessinArcPSM(P, S, M) {
    context.strokeStyle = couleurArcs;
    var PEq = new Vecteur3D(1, 0, S.get(2)); // P dans le repère équatorial, avec le même phi que S
    dessinArc(PEq, S.get(1) < M.get(1) ? S : M, true, false);
}

//protected void dessinAngleDelta(final Vecteur M, final Vecteur S) {
function dessinAngleDelta(M, S) {
    context.strokeStyle = couleurDelta;
    context.fillStyle = couleurDelta;
    dessinArc(M, S, true, true);
    if (Math.abs(delta) > 0.1) {
        var r;
        if ((phi >= 0 && delta >= 0) || (phi < 0 && delta < 0))
            r = 1.1;
        else
            r = 0.9;
        var pt = conversion(new Vecteur3D(r, Math.PI/2 - (Math.PI/2 - S.get(1))/2, M.get(2)), true);
        context.fillText(titreDelta, pt.x - context.measureText(titreDelta).width/2, pt.y + 5);
    }
}

//protected void dessinAnglePhi(final Vecteur P) {
function dessinAnglePhi(P) {
    context.strokeStyle = couleurPhi;
    context.fillStyle = couleurPhi;
    dessinArc(Nord, P, false, true);
    var pt = conversion(new Vecteur3D(1.05, Math.PI/2 - phi/2, -Math.PI/2), false);
    context.fillText(titrePhi, pt.x - context.measureText(titrePhi).width/2, pt.y + 5);
}

//protected void dessinAngleA(final Vecteur N) {
function dessinAngleA(N) {
    context.strokeStyle = couleurAngleA;
    context.fillStyle = couleurAngleA;
    dessinArc(Sud, N, false, true);
    var pt = conversion(new Vecteur3D(0.87, Math.PI/2, Math.PI/2 - (Math.PI/2 - N.get(2))/2), false);
    context.fillText('A', pt.x - 10, pt.y + 5);
}

//protected void dessinAngleH(final Vecteur M) {
function dessinAngleH(M) {
    context.strokeStyle = couleurAngleH;
    context.fillStyle = couleurAngleH;
    dessinArc(new Vecteur3D(1, Math.PI/2, Math.PI/2), M, true, true);
    var pt = conversion(new Vecteur3D(0.87, Math.PI/2, Math.PI/2 - (Math.PI/2 - M.get(2))/2), true);
    context.fillText('H', pt.x - 10, pt.y + 5);
}

//protected void dessinAngleHauteur(final Vecteur N, final Vecteur SsphHoz) {
function dessinAngleHauteur(N, SsphHoz) {
    context.strokeStyle = couleurAngleHauteur;
    context.fillStyle = couleurAngleHauteur;
    dessinArc(N, SsphHoz, false, true);
    if (Math.abs(SsphHoz.get(1) - Math.PI/2) > 0.1) {
        var r;
        if ((phi >= 0 && delta >= 0) || (phi < 0 && delta < 0))
            r = 0.9;
        else
            r = 1.1;
        var pt = conversion(new Vecteur3D(r, Math.PI/2 - (Math.PI/2 - SsphHoz.get(1))/2, N.get(2)), false);
        context.fillText('h', pt.x - 5, pt.y + 5);
    }
}



// dessin d'un arc entre 2 points dans l'un des 2 systèmes de coordonnées sphériques, en suivant l'une des coordonnées
//protected void dessinArc(final Vecteur v1, final Vecteur v2, final boolean coordEq, final boolean fleche)
function dessinArc(v1, v2, coordEq, fleche) {
    var nb = precisionCourbes/4;
    var dr = (v2.get(0) - v1.get(0)) / nb;
    var dtheta = (v2.get(1) - v1.get(1)) / nb;
    var dphi = (v2.get(2) - v1.get(2)) / nb; // ne marche pas toujours (par ex. avec theta = 0)
    context.beginPath();
    var pt = conversion(v1, coordEq);
    var vpt = null;
    context.moveTo(pt.x, pt.y);
    for (var i=1; i<=nb; i++) {
        if (i == nb)
            vpt = pt;
        pt = conversion(new Vecteur3D(v1.get(0) + i * dr, v1.get(1) + i * dtheta, v1.get(2) + i * dphi), coordEq);
        context.lineTo(pt.x, pt.y);
    }
    context.stroke();
    if (fleche && Math.abs(v2.get(0) - v1.get(0)) + Math.abs(v2.get(1) - v1.get(1)) + Math.abs(v2.get(2) - v1.get(2)) > 0.1)
        dessinFleche(vpt, pt);
}

// dessin des 2 traits de la fin de la flêche à partir des 2 derniers points d'un arc
//protected void dessinFleche(final Point2D pt1, final Point2D pt2)
function dessinFleche(pt1, pt2) {
    var l = 6; // longueur des traits en pixels
    var da = Math.PI / 8; // angle des traits avec la fin de l'arc
    var dx = pt2.x - pt1.x;
    var dy = pt2.y - pt1.y;
    var angle;
    if (dx == 0) {
        if (dy > 0)
            angle = Math.PI / 2;
        else
            angle = -Math.PI / 2;
    } else {
        angle = Math.atan(dy / dx);
        if (dx < 0)
            angle = angle + Math.PI;
    }
    var ptFleche = new Point2D(pt2.x - l * Math.cos(angle + da), pt2.y - l * Math.sin(angle + da));
    context.beginPath();
    context.moveTo(pt2.x, pt2.y);
    context.lineTo(ptFleche.x, ptFleche.y);
    context.stroke();
    ptFleche = new Point2D(pt2.x - l * Math.cos(angle - da), pt2.y - l * Math.sin(angle - da));
    context.beginPath();
    context.moveTo(pt2.x, pt2.y);
    context.lineTo(ptFleche.x, ptFleche.y);
    context.stroke();
}

// passage des coordonnées sphériques aux coordonnées écran
//protected Point2D conversion(final Vecteur vsph, final boolean rotEq)
function conversion(vsph, rotEq) {
    // passage aux coordonnées cartésiennes
    var v = vsph.spheriquesVersCartesiennes();
    
    // rotation optionelle de l'équateur et rotation permettant de voir l'horizon de haut
    if (rotEq)
        v = rotObsEq.produitV(v);
    else
        v = rotObservateur.produitV(v);
    
    return(projectionEcran(v));
}

// ajout d'un point en coordonnées sphériques (r, theta, phi)
//protected void afficherPoint(final Vecteur vsph, final boolean rotEq, final boolean croix, final String titre)
function afficherPoint(vsph, rotEq, croix, titre) {
    var p = conversion(vsph, rotEq);
    
    // affichage d'une croix ou d'un point
    if (croix) {
        context.beginPath();
        context.moveTo(p.x - 2, p.y);
        context.lineTo(p.x + 2, p.y);
        context.stroke();
        context.beginPath();
        context.moveTo(p.x, p.y - 2);
        context.lineTo(p.x, p.y + 2);
        context.stroke();
    } else
        context.beginPath();
        context.moveTo(p.x, p.y);
        context.lineTo(p.x, p.y);
        context.stroke();
    
    // affichage du titre
    if (titre != null) {
        var l = context.measureText(titre).width;
        context.fillText(titre, p.x - l/2, p.y - 9);
    }
}

// projection selon l'axe x à partir des coordonnées cartésiennes, puis conversion vers l'écran
//protected Point2D projectionEcran(final Vecteur v)
function projectionEcran(v) {
    return(new Point2D(cx + v.get(1) * rayon, cy - v.get(2) * rayon));
}

// affichage en degrés minutes d'un angle en radians
//protected static String affichageAngleDMS(final double a, final boolean estOuest, final boolean angle360)
function affichageAngleDMS(a, estOuest, angle360) {
    var d = Math.toDegrees(Math.abs(a));
    var id = Math.floor(d);
    var m = (d - id)*60;
    var im = Math.round(m);
    if (im >= 60) {
        im -= 60;
        id += 1;
    }
    var signe;
    if (angle360 || a == 0)
        signe = '';
    else if (a > 0)
        signe = '+';
    else
        signe = '-';
    if (estOuest)
        return(id + '° ' + im + "'" + (a >= 0 ? ' E' : ' O'));
    else
        return(signe + id + '° ' + im + "'");
}


function Matrice(lignes, colonnes) {
    // m = new double[colonnes][lignes]
    this.m = new Array(colonnes);
    for (var i=0; i<colonnes; i++)
        this.m[i] = new Array(lignes);
    return(this);
}
//public Vecteur produit(final Vecteur v)
Matrice.prototype.produitV = function(v) {
    var nbc = this.m.length;
    var nbl = this.m[0].length;
    if (nbc != v.getDimensions()) {
        alert("produit impossible d'une matrice avec un vecteur (" + nbc + " != " + v.getDimensions() + ")");
        return(null);
    }
    var res = new Vecteur(nbl);
    var d;
    for (var j=0; j<nbl; j++) {
        d = 0;
        for (var i=0; i<nbc; i++)
            d += v.get(i) * this.m[i][j];
        res.set(j, d);
    }
    return(res);
}
//public Matrice produit(final Matrice mat)
Matrice.prototype.produitM = function(mat) {
    var nbc = this.m.length;
    var nbl = this.m[0].length;
    if (nbc != mat.getLignes()) {
        alert("produit impossible d'une matrice avec une matrice (" + nbc + " != " + mat.getLignes() + ")");
        return(null);
    }
    var res = new Matrice(nbl, nbc);
    var d;
    for (var jr=0; jr<nbc; jr++) {
        for (var ir=0; ir<nbl; ir++) {
            d = 0;
            for (var i=0; i<nbc; i++)
                d += this.m[i][jr] * mat.get(ir, i);
            res.set(ir, jr, d);
        }
    }
    return(res);
}
//public int getColonnes()
Matrice.prototype.getColonnes = function() {
    return(this.m.length);
}
//public int getLignes()
Matrice.prototype.getLignes = function() {
    return(this.m[0].length);
}
//public double get(final int colonne, final int ligne)
Matrice.prototype.get = function(colonne, ligne) {
    return(this.m[colonne][ligne]);
}
//public void set(final int colonne, final int ligne, final double valeur)
Matrice.prototype.set = function(colonne, ligne, valeur) {
    this.m[colonne][ligne] = valeur;
}


//public Vecteur(final int dimensions)
function Vecteur(dimensions) {
    // v = new double[dimensions];
    if (typeof(dimensions) == 'undefined')
        this.v = null;
    else
        this.v = new Array(dimensions);
    return(this);
}
//public Vecteur(final double v1, final double v2, final double v3)
Vecteur3D.prototype = new Vecteur();
Vecteur3D.prototype.constructor = Vecteur3D;
function Vecteur3D(v1, v2, v3) {
    Vecteur.call(this, 3);
    this.v = [v1, v2, v3];
    return(this);
}
//public int getDimensions()
Vecteur.prototype.getDimensions = function() {
    return(this.v.length);
}
//public double get(final int pos)
Vecteur.prototype.get = function(pos) {
    return(this.v[pos]);
}
//public void set(final int pos, final double valeur)
Vecteur.prototype.set = function(pos, valeur) {
    this.v[pos] = valeur;
}
// coordonnées sphériques (r, theta, phi) vers cartésiennes (x, y, z)
//public Vecteur spheriquesVersCartesiennes()
Vecteur.prototype.spheriquesVersCartesiennes = function() {
    if (this.v.length != 3) {
        alert("Conversion impossible de coordonnées (" + this.v.length + " != 3)");
        return(null);
    }
    var res = new Vecteur(3);
    res.set(0, this.v[0] * Math.sin(this.v[1]) * Math.cos(this.v[2]));
    res.set(1, this.v[0] * Math.sin(this.v[1]) * Math.sin(this.v[2]));
    res.set(2, this.v[0] * Math.cos(this.v[1]));
    return(res);
}
// coordonnées cartésiennes (x, y, z) vers sphériques (r, theta, phi)
//public Vecteur cartesiennesVersSpheriques()
Vecteur.prototype.cartesiennesVersSpheriques = function() {
    if (this.v.length != 3) {
        System.err.println("Conversion impossible de coordonnées (" + this.v.length + " != 3)");
        return(null);
    }
    var res = new Vecteur(3);
    var r = Math.sqrt(this.v[0]*this.v[0] + this.v[1]*this.v[1] + this.v[2]*this.v[2]);
    res.set(0, r);
    res.set(1, Math.acos(this.v[2]/r));
    if (this.v[0] > 0) // x > 0
        res.set(2, Math.atan(this.v[1]/this.v[0]));
    else if (this.v[0] < 0) // x < 0
        res.set(2, Math.PI + Math.atan(this.v[1]/this.v[0]));
    else if (this.v[1] > 0) // x = 0, y > 0
        res.set(2, Math.PI / 2);
    else if (this.v[1] < 0) // x = 0, y < 0
        res.set(2, 3 * Math.PI / 2);
    else // x = 0, y = 0 : peu importe !
        res.set(2, 0);
    return(res);
}

function Point2D(x, y) {
    this.x = x;
    this.y = y;
}

